package com.subd.subd.Controllers;

import com.subd.subd.Dtos.OrderDto;
import com.subd.subd.Models.Order;
import com.subd.subd.Services.OrderService;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

@RestController
public class OrderController {
    private final OrderService OrderService;
    public OrderController(OrderService OrderService) {
        this.OrderService = OrderService;
    }
    @GetMapping("/order/{id}")
    public Order getOrder(@PathVariable Long id) {
        return OrderService.findOrder(id);
    }
    @GetMapping("/order")
    public List<Order> getOrders() {
        return OrderService.findAllOrders();
    }
    @GetMapping("/order/filter")
    public List<Order> getFilteredOrders(@RequestParam(value = "id", required = false) Long id,
                                         @RequestParam(value = "value", required = false) Double value,
                                         @RequestParam(value = "status", required = false) String status,
                                         @RequestParam(value = "date", required = false) Date date,
                                         @RequestParam(value = "clientId", required = false) Long clientId,
                                         @RequestParam(value = "sourcePickUpPointId", required = false) Long sourcePickUpPointId,
                                         @RequestParam(value = "destPickUpPointId", required = false) Long destPickUpPointId,
                                         @RequestParam(value = "carId", required = false) Long carId) {
        return OrderService.findfilteredOrders(id, value, status, date, clientId, sourcePickUpPointId, destPickUpPointId, carId);
    }
    @PostMapping("/order")
    public Order createOrder(@RequestBody OrderDto orderDto) {
        return OrderService.addOrder(orderDto.getValue(), orderDto.getStatus(), orderDto.getDate(), orderDto.getClientId(), orderDto.getSourcePickUpPointId(), orderDto.getDestPickUpPointId(), orderDto.getCarId());
    }
    @PutMapping("/order/{id}")
    public Order updateOrder(@PathVariable Long id, @RequestBody OrderDto orderDto) {
        return OrderService.updateOrder(id, orderDto.getValue(), orderDto.getStatus(), orderDto.getDate(), orderDto.getClientId(), orderDto.getSourcePickUpPointId(), orderDto.getDestPickUpPointId(), orderDto.getCarId());
    }
    @DeleteMapping("/order/{id}")
    public Order deleteOrder(@PathVariable Long id) {
        return OrderService.deleteOrder(id);
    }
}
