package com.subd.subd.Services;

import com.subd.subd.Exceptions.PickUpPointNotFoundException;
import com.subd.subd.Models.PickUpPoint;
import com.subd.subd.Repositories.PickUpPointRepository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
public class PickUpPointService {
    private final PickUpPointRepository pickUpPointRepository;
    public PickUpPointService(PickUpPointRepository pickUpPointRepository) {
        this.pickUpPointRepository = pickUpPointRepository;
    }
    @Transactional
    public PickUpPoint addPickUpPoint(String address) {
        final PickUpPoint pickUpPoint = new PickUpPoint(address);
        return pickUpPointRepository.save(pickUpPoint);
    }
    @Transactional(readOnly = true)
    public PickUpPoint findPickUpPoint(Long id) {
        final Optional<PickUpPoint> pickUpPoint = pickUpPointRepository.findById(id);
        return pickUpPoint.orElseThrow(() -> new PickUpPointNotFoundException(id));
    }
    @Transactional(readOnly = true)
    public List<PickUpPoint> findAllPickUpPoints() {
        return pickUpPointRepository.findAll();
    }
    @Transactional
    public PickUpPoint updatePickUpPoint(Long id, String address) {
        final PickUpPoint currentPickUpPoint = findPickUpPoint(id);
        if (address != null) {
            currentPickUpPoint.setAddress(address);
        }
        return pickUpPointRepository.save(currentPickUpPoint);
    }
    @Transactional
    public PickUpPoint deletePickUpPoint(Long id) {
        final PickUpPoint currentPickUpPoint = findPickUpPoint(id);
        pickUpPointRepository.delete(currentPickUpPoint);
        return currentPickUpPoint;
    }
    @Transactional
    public void deleteAllPickUpPoints() {
        pickUpPointRepository.deleteAll();
    }
}
